/*
*  Licensed Materials - Property of IBM
*  
*  CICS SupportPac CA1T
* 
*   Copyright IBM Corporation 2008
*  All Rights Reserved
*  
*  US Government Users Restricted Rights  Use,
*  duplication, or disclosure restricted by GSA ADP
*  Schedule Contract with IBM Corporation
 */

/*
 *      File Name:    FailOverExit.java
 *      Version:      1.0 
 *      Date:         170510
 *      Product:      CICS Transaction Gateway for z/OS
 * 
 * Description: The FailOver exit is designed to redirect and retry an ECI request across
 * a group of cloned servers
 * 
 * See SupportPac documentation for further details on how to configure and use
 * this sample
 */
package com.ibm.ctg.samples.ha;


import java.util.Hashtable;
import java.util.Map;

import com.ibm.ctg.ha.CICSRequestExit;
import com.ibm.ctg.ha.RequestDetails;
import com.ibm.jzos.ZUtil;


public class FailOverExit implements CICSRequestExit {
	//                                             
	public static final String copyright = "\nLicensed Materials - Property of IBM "+
	"CICS SupportPac CA1T" +
	"(c) Copyright IBM Corp. 2008, 2010  All Rights Reserved. " +
	"US Government Users Restricted Rights - Use, duplication or " +
	"disclosure restricted by GSA ADP Schedule Contract with " +
	"IBM Corp.\n";
	//


    private static String jobname; // CTG job name
    
    private Hashtable<String, String[]> tables ; // Table of servers

    private String[] serverArray; // server array for HA

    private int len; // Number of servers in HA serverArray

    HAUtil util = new HAUtil(); //util object

    /**
     * Constructor called at exit initialisation by Gateway 
     */
    public FailOverExit() throws Exception {


        try {

            jobname = ZUtil.getCurrentJobname(); // CTG jobname
            util.getstdout();   // Setup output destination

            util.println("Starting CICS request exit " 
            		+ getClass().getSimpleName()
            		+ " V"+ HAUtil.VERSION
            		+ " in Gateway " + jobname);

            tables = util.readfile(util.openfile()); // Build the hash table from the config file

            serverArray = tables.get(jobname); // Build the server list for this CTG
            if (serverArray != null) {
                util.traceln("Tables built succesfully ");
                len = serverArray.length;
            } else {
                throw new Exception ("Jobname not found in configuration file");
            }

            StringBuffer msg = new StringBuffer();
            msg.append("Server selection for " + jobname + " built with servers: ");
            int i = 0;
            while (i < len) {
                msg.append(serverArray[i] + ",");
                i++;
            }
            util.println(msg.toString());

        } catch (Exception e) {
            util.println ("Terminating with " + e.toString());
            throw new Exception (e);
        }
    }

    /**
     * Constructor called at exit initialisation by Gateway 
     */
    public String getCICSServer(Map<RequestDetails, Object> data) {

        Object dataItem;
        String server; // input server on ECI request
        String cicsserver = ""; // Server to be returned from exit



        dataItem = data.get(RequestDetails.Server);
        if (dataItem != null) {
            server = data.get(RequestDetails.Server).toString();
        } else {
            server = "null";
        }

        //Check if we have a server list 
        if (serverArray != null) {
            //Retrieve how many times we've attempted this request
            int index = (Integer) data.get(RequestDetails.RetryCount);
            //Ensure that we're not going to index beyond the end of the array
            if (index > serverArray.length - 1) {
                index = serverArray.length - 1;
            }
            //Retrieve the CICS server that the request should go to
            cicsserver = serverArray[index];
        }

        util.traceln (" Retry:" + data.get(RequestDetails.RetryCount)+
                      " remapped server:" + server  +
                      " to:" + cicsserver);


        //Return the name of the CICS server from the HA list
        return cicsserver;
    }

    /**
     * Called at Gateway startup to set the max number of retries the Gateway will issue for an ECI
     * request if a request fails
     * 
     * Returns retry count 
     */
    public int getRetryCount() {

        //retry enough times so that all items in the table are used 
        return len - 1;
    }


    /**
     * 
     * Returns name of the current class
     */
    public String myName() {
        return this.getClass().getCanonicalName();
    }

    /**
     * Called at shutdwon of the Gateway
     */
    public void shutdown() {

        util.println("Shutdown initiated in " + getClass().getSimpleName());
    }


}

