/*
*  Licensed Materials - Property of IBM
*  
*  CICS SupportPac CA1T
* 
*   Copyright IBM Corporation 2008,2010
*  All Rights Reserved
*  
*  US Government Users Restricted Rights  Use,
*  duplication, or disclosure restricted by GSA ADP
*  Schedule Contract with IBM Corporation
*/

/*
 *      File Name:    HAUtil.java
 *      Version:      1.1
 *      Date:         170510
 *      Product:      CICS Transaction Gateway for z/OS
 * 
 *      Utility class for CICS Transaction Gatway SupportPac CA1T
 *      Provides constant values and common methods
 * 
 */

package com.ibm.ctg.samples.ha;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.LineNumberReader;
import java.io.PrintStream;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Hashtable;
import java.util.StringTokenizer;

import com.ibm.ctg.client.T;
import com.ibm.jzos.FileFactory;

public class HAUtil {
	//                                             
	public static final String copyright = "\nLicensed Materials - Property of IBM "+
	"CICS SupportPac CA1T" +
	"(c) Copyright IBM Corp. 2008, 2010  All Rights Reserved. " +
	"US Government Users Restricted Rights - Use, duplication or " +
	"disclosure restricted by GSA ADP Schedule Contract with " +
	"IBM Corp.\n";
	//

    PrintStream stdout; // Print stream for output

    static final String ENV_FILE = "CTG_HACONFIG"; // Env. variable for onfig file 

    static final String id = " CA1T "; // SupportPac ID
    public static final String VERSION = "1.1";

    static final String comma = ","; // index delimiter in config file

    static final String delim = "="; // key delimiter in config file

    static final String STDOUT_PROPERTY = "com.ibm.ctg.samples.cicsrequestexit.out"; // logging destination

    static final String dateFmt = "MM/dd/yy HH:mm:ss:SSS"; // Format of time stamp for msgs


    /**
     * Setup the stdout logging destination for message
     * Default is Gateway STDOUT, but can be set to DD
     */
    public void getstdout ()
    {
        // set up output destination
        String destination = System.getProperty(STDOUT_PROPERTY);
        if (destination == null) {
            stdout = getPrintStream();
        } else {

            File destFile = new File(destination);

            // Use com.ibm.jzos.FileFactory to handle DD output
            try {
                if (destination.substring(0, 2).equalsIgnoreCase("//")) {
                    BufferedOutputStream bufOut = FileFactory.newBufferedOutputStream(destination);
                    stdout = new PrintStream(bufOut);
                } else {
                    stdout = new PrintStream(destFile);
                }
            } catch (FileNotFoundException e) {
                stdout = getPrintStream();
                System.err.println("Print to '" + destination
                                   + "' failed with " + e);
                System.err.println("Using default PrintStream");
            } catch (IOException e) {
                stdout = getPrintStream();
                System.err.println("Print to '" + destination
                                   + "' failed with " + e);
                System.err.println("Using default PrintStream");
            }
        }
    }

    /**
     * Returns PrintStream to use as default output logging devic
     */
    public PrintStream getPrintStream() {
        return System.out;
    }

    /**
     * Open config file
     */
    public String openfile() throws Exception {

        try {
            String configfile = System.getenv(ENV_FILE);
            if (configfile == null) {
                throw new IOException(ENV_FILE + " variable not set");
            }

            println("Reading file " + configfile);

            return configfile;


        } catch (FileNotFoundException fe) {
            println ("Error file not found " + fe.toString());
            throw new Exception (fe);
        } catch (IOException ioe) {
            println ("Error reading file " + ioe.toString());
            throw new Exception (ioe);
        }

    }

    /**
     * Send a string to current PrintWriter
     */
    public void print( String args) {

        try {
            stdout.print(args);

        } catch (Throwable t) {
            // We cannot output anything so throw exception to disable exit
            throw new RuntimeException("Default PrintStream has failed!");

        }

    }

    /**
     * Send a string to the PrintWriter, append a newline, and flush
     */
    public void println( String string) {

        if ( string != null ) {
            Date timestamp = new Date();
            SimpleDateFormat dfTime = null;
            dfTime = (SimpleDateFormat) DateFormat.getTimeInstance();
            dfTime.applyPattern(dateFmt);
            String time = dfTime.format(timestamp);

            print(time + id + string + "\n");
            if (stdout == null) {
                System.out.flush();
            } else {
                stdout.flush();
            }
        }
    }

    /**
     * Output routine to trace a string to our currently set PrintWriter
     */
    public void traceln(String string) {
        if ((string != null) && (T.getTraceOn())) {
            println(string);
        }
    }

    /**
     * Read and parse configuration file
     * File is of the format KEY=VALUE,VALUE,VALUE
     * 
     * Returns Hash table, containing an array of values in each row
     */

    public Hashtable<String, String[]> readfile (String file) throws Exception {

        String strLine; // Line to be read
        String value; // String of server values
        String array[];
        FileReader frProperties = new FileReader(file);
        LineNumberReader lnrProperties = new LineNumberReader(frProperties);
        Hashtable<String, String[]> tables = new Hashtable<String, String[]>(); // Table of servers

        try {

            // loop through file until null line is read
            while (((strLine = lnrProperties.readLine()) != null)) {

                strLine = strLine.trim();// Trim  whitespace
                if ((strLine.length() > 0) && !strLine.startsWith("#")) {
                    String key; // key
                    int keyEnd = strLine.indexOf(delim); // find KEY
                    if ((keyEnd > 0 ) && (keyEnd < strLine.length())) {
                        key = strLine.substring(0,keyEnd).toUpperCase();
                    } else {
                        throw new IOException("Key parsing failed");
                    }
                    //Parse the rest of the line to read list of values
                    value = strLine.substring(keyEnd+1,strLine.length()).toUpperCase();
                    traceln("key = " + key + " value = " + value);

                    StringTokenizer tokenizer = new StringTokenizer(value,comma);
                    int count = tokenizer.countTokens();  // count the number of values 
                    array = new String[count]; //initialise array	

                    int c = 0;
                    while (tokenizer.hasMoreTokens()) {
                        array[c] = tokenizer.nextToken();
                        traceln("Added value " + array[c].toString()); 
                        c++;
                    }  
                    tables.put(key, array); // build hash table of key=values					
                }
            }

        } catch (FileNotFoundException e) {
            throw e;
        } catch (IOException e) {
            throw e;
        } catch (Exception e) {
            e.printStackTrace();

        }
        return tables;

    }



}
